==> fxt-2008.02.02.tgz: fxt/src/bits/bitlow.h <==
#if !defined HAVE_BITLOW_H__
#define      HAVE_BITLOW_H__

#include "fxttypes.h"
#include "bits/bitsperlong.h"
#include "bits/bitasm.h"


static inline ulong lowest_bit_01edge(ulong x)
// Return word where a all bits from  (including) the
//   lowest set bit to bit 0 are set.
// Return 0 if no bit is set.
{
    if ( 0==x )  return 0;
    return  x^(x-1);
}
// -------------------------

static inline ulong lowest_bit_10edge(ulong x)
// Return word where a all bits from (including) the
//   lowest set bit to most significant bit are set.
// Return 0 if no bit is set.
{
    if ( 0==x )  return 0;
    x ^= (x-1);
    // here  x == lowest_bit_01edge(x);
    return  ~(x>>1);
}
// -------------------------


static inline ulong lowest_bit_idx(ulong x)
// Return index of lowest bit set.
// Bit index ranges from zero to BITS_PER_LONG-1.
// Examples:
//    ***1 --> 0
//    **10 --> 1
//    *100 --> 2
// Return 0 (also) if no bit is set.
{
#if defined  BITS_USE_ASM
    return  asm_bsf(x);
#else // BITS_USE_ASM

//    if ( 1>=x )  return  x-1; // 0 if 1, ~0 if 0
//    if ( 0==x )  return  0;

    ulong r = 0;
    x &= -x;  // isolate lowest bit
#if  BITS_PER_LONG >= 64
    if ( x & 0xffffffff00000000UL )  r += 32;
    if ( x & 0xffff0000ffff0000UL )  r += 16;
    if ( x & 0xff00ff00ff00ff00UL )  r += 8;
    if ( x & 0xf0f0f0f0f0f0f0f0UL )  r += 4;
    if ( x & 0xccccccccccccccccUL )  r += 2;
    if ( x & 0xaaaaaaaaaaaaaaaaUL )  r += 1;
#else // BITS_PER_LONG >= 64
    if ( x & 0xffff0000UL )  r += 16;
    if ( x & 0xff00ff00UL )  r += 8;
    if ( x & 0xf0f0f0f0UL )  r += 4;
    if ( x & 0xccccccccUL )  r += 2;
    if ( x & 0xaaaaaaaaUL )  r += 1;
#endif // BITS_PER_LONG >= 64

    return r;
#endif // BITS_USE_ASM
}
// -------------------------

static inline ulong lowest_bit_idx_parity(ulong x)
// Return the parity of the index of the lowest set bit.
// Return zero with x=0.
{
    x &= -x;  // isolate lowest bit
#if BITS_PER_LONG == 64
    return  (x & 0xaaaaaaaaaaaaaaaaUL);
#else
    return  (x & 0xaaaaaaaaUL);
#endif
}
// -------------------------


static inline ulong lowest_bit(ulong x)
// Return word where only the lowest set bit in x is set.
// Return 0 if no bit is set.
{
//    if ( 0==x )  return 0;
//    return  ((x^(x-1)) >> 1) + 1;

//    return  (x & (x-1)) ^ x;

    return  x & -x;  // use: -x == ~x + 1
}
// -------------------------

static inline ulong lowest_zero(ulong x)
// Return word where only the lowest unset bit in x is set.
// Return 0 if all bits are set.
{
//    return  (x ^ (x+1)) & ~x;
//    return  ((x ^ (x+1)) >> 1 ) + 1;

    x = ~x;
    return  x & -x;
}
// -------------------------

static inline ulong lowest_block(ulong x)
// Isolate lowest block of ones.
// e.g.:
// x   = *****011100
// l   = 00000000100
// y   = *****100000
// x^y = 00000111100
// ret = 00000011100
{
    ulong l = x & -x;  // lowest bit
    ulong y = x + l;
    y ^= x;
    return  y & (y>>1);
}
// -------------------------

static inline ulong delete_lowest_bit(ulong x)
// Return word were the lowest bit set in x is cleared.
// Return 0 for input == 0.
{
    return  x & (x-1);
}
// -------------------------

static inline ulong set_lowest_zero(ulong x)
// Return word were the lowest unset bit in x is set.
// Return ~0 for input == ~0.
{
    return  x | (x+1);
}
// -------------------------


static inline ulong low_bits(ulong x)
// Return word where all the (low end) ones are set.
// Example:  01011011 --> 00000011
// Return 0 if lowest bit is zero:
//       10110110 --> 0
{
    if ( ~0UL==x )  return ~0UL;
    return (((x+1)^x) >> 1);
}
// -------------------------

static inline ulong low_zeros(ulong x)
// Return word where all the (low end) zeros are set.
// Example:  01011000 --> 00000111
// Return 0 if all bits are set.
{
    if ( 0==x )  return ~0UL;
    return (((x-1)^x) >> 1);
}
// -------------------------


#endif  // !defined HAVE_BITLOW_H__

==> fxt-2009.02.02.tgz: fxt/src/bits/bitlow.h <==
#if !defined HAVE_BITLOW_H__
#define      HAVE_BITLOW_H__

#include "fxttypes.h"
#include "bits/bitsperlong.h"
#include "bits/bitasm.h"


static inline ulong lowest_one_01edge(ulong x)
// Return word where all bits from  (including) the
//   lowest set bit to bit 0 are set.
// Return 0 if no bit is set.
{
    if ( 0==x )  return 0;
    return  x^(x-1);
}
// -------------------------

static inline ulong lowest_one_10edge(ulong x)
// Return word where all bits from (including) the
//   lowest set bit to most significant bit are set.
// Return 0 if no bit is set.
{
    if ( 0==x )  return 0;
    x ^= (x-1);
    // here  x == lowest_one_01edge(x);
    return  ~(x>>1);
}
// -------------------------


static inline ulong lowest_one_idx(ulong x)
// Return index of lowest bit set.
// Bit index ranges from zero to BITS_PER_LONG-1.
// Examples:
//    ***1 --> 0
//    **10 --> 1
//    *100 --> 2
// Return 0 (also) if no bit is set.
{
#if defined  BITS_USE_ASM
    return  asm_bsf(x);
#else // BITS_USE_ASM

//    if ( 1>=x )  return  x-1; // 0 if 1, ~0 if 0
//    if ( 0==x )  return  0;

    ulong r = 0;
    x &= -x;  // isolate lowest bit
#if  BITS_PER_LONG >= 64
    if ( x & 0xffffffff00000000UL )  r += 32;
    if ( x & 0xffff0000ffff0000UL )  r += 16;
    if ( x & 0xff00ff00ff00ff00UL )  r += 8;
    if ( x & 0xf0f0f0f0f0f0f0f0UL )  r += 4;
    if ( x & 0xccccccccccccccccUL )  r += 2;
    if ( x & 0xaaaaaaaaaaaaaaaaUL )  r += 1;
#else // BITS_PER_LONG >= 64
    if ( x & 0xffff0000UL )  r += 16;
    if ( x & 0xff00ff00UL )  r += 8;
    if ( x & 0xf0f0f0f0UL )  r += 4;
    if ( x & 0xccccccccUL )  r += 2;
    if ( x & 0xaaaaaaaaUL )  r += 1;
#endif // BITS_PER_LONG >= 64

    return r;
#endif // BITS_USE_ASM
}
// -------------------------

static inline ulong lowest_one_idx_parity(ulong x)
// Return the parity of the index of the lowest set bit.
// Return zero with x=0.
{
    x &= -x;  // isolate lowest bit
#if BITS_PER_LONG == 64
    return  0 != (x & 0xaaaaaaaaaaaaaaaaUL);
#else
    return  0 != (x & 0xaaaaaaaaUL);
#endif
}
// -------------------------


static inline ulong lowest_one(ulong x)
// Return word where only the lowest set bit in x is set.
// Return 0 if no bit is set.
{
//    if ( 0==x )  return 0;
//    return  ((x^(x-1)) >> 1) + 1;

//    return  (x & (x-1)) ^ x;

    return  x & -x;  // use: -x == ~x + 1
}
// -------------------------

static inline ulong lowest_zero(ulong x)
// Return word where only the lowest unset bit in x is set.
// Return 0 if all bits are set.
{
//    return  (x ^ (x+1)) & ~x;
//    return  ((x ^ (x+1)) >> 1 ) + 1;

    x = ~x;
    return  x & -x;
}
// -------------------------

static inline ulong lowest_block(ulong x)
// Isolate lowest block of ones.
// e.g.:
// x   = *****011100
// l   = 00000000100
// y   = *****100000
// x^y = 00000111100
// ret = 00000011100
{
    ulong l = x & -x;  // lowest bit
    ulong y = x + l;
    y ^= x;
    return  y & (y>>1);
}
// -------------------------

static inline ulong delete_lowest_one(ulong x)
// Return word where the lowest bit set in x is cleared.
// Return 0 for input == 0.
{
    return  x & (x-1);
}
// -------------------------

static inline ulong set_lowest_zero(ulong x)
// Return word where the lowest unset bit in x is set.
// Return ~0 for input == ~0.
{
    return  x | (x+1);
}
// -------------------------


static inline ulong low_ones(ulong x)
// Return word where all the (low end) ones are set.
// Example:  01011011 --> 00000011
// Return 0 if lowest bit is zero:
//       10110110 --> 0
{
    x = ~x;
    x &= -x;
    --x;
    return x;

//    if ( ~0UL==x )  return ~0UL;
//    return (((x+1)^x) >> 1);
}
// -------------------------

static inline ulong low_zeros(ulong x)
// Return word where all the (low end) zeros are set.
// Example:  01011000 --> 00000111
// Return 0 if all bits are set.
{
    x &= -x;
    --x;
    return x;

//    if ( 0==x )  return ~0UL;
//    return (((x-1)^x) >> 1);
}
// -------------------------


#endif  // !defined HAVE_BITLOW_H__
